'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol ? "symbol" : typeof obj; };

var _slicedToArray = function () { function sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"]) _i["return"](); } finally { if (_d) throw _e; } } return _arr; } return function (arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }; }();

var lang = {};

exports.default = lang;
var coerce = exports.coerce = lang.coerce = {};

coerce.integer = function (v) {
  return parseInt(v, 10);
};
coerce.string = String;
coerce.bool = function (v) {
  return !empty(coerce.integer(v));
};
coerce.number = Number;

/**
 * Given an Object or Map, return a list of two-element tuples. Each tuple is
 * an Array that contains the key at index 0, and the input at index 1.
 * @param {Object|Map} input
 * @return {Array}
 */
var tuples = exports.tuples = lang.tuples = function (input) {
  var output = [];

  if (input instanceof Map) {
    var _iteratorNormalCompletion = true;
    var _didIteratorError = false;
    var _iteratorError = undefined;

    try {
      for (var _iterator = input.entries()[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
        var _step$value = _slicedToArray(_step.value, 2);

        var k = _step$value[0];
        var v = _step$value[1];

        output.push([k, v]);
      }
    } catch (err) {
      _didIteratorError = true;
      _iteratorError = err;
    } finally {
      try {
        if (!_iteratorNormalCompletion && _iterator.return) {
          _iterator.return();
        }
      } finally {
        if (_didIteratorError) {
          throw _iteratorError;
        }
      }
    }
  } else {
    for (var prop in input) {
      if (input.hasOwnProperty(prop)) {
        output.push([prop, input[prop]]);
      }
    }
  }

  return output;
};

/**
 * This works like PHP's empty() function. Behaviors:
 *  - return true if value is falsy
 *  - return true if the value is the integer zero
 *  - return true if the value has a length property that is zero
 *  - return true if the value is an object that does not own any properties
 *  - return false for everything else
 * @param mixed value
 * @return {Boolean}
 */
var empty = exports.empty = lang.empty = function (value) {
  if (value === undefined) {
    return true;
  }

  if ((typeof value === 'undefined' ? 'undefined' : _typeof(value)) !== 'object' && isNaN(value)) {
    return false;
  }

  // falsy value is always empty
  if (!value) {
    return true;
  }

  // number zero is empty
  if (typeof value === 'number') {
    return value === 0;
  }

  // zero-length anything is empty
  if (typeof value.length === 'number') {
    return !value.length;
  }

  // zero-size anything is empty
  if (typeof value.size === 'number') {
    return !value.size;
  }

  // an object that does not own any properties is empty
  if ((typeof value === 'undefined' ? 'undefined' : _typeof(value)) === 'object') {
    return Object.getOwnPropertyNames(value).length === 0;
  }

  // default to false
  return false;
};