'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.Resource = exports.resources = exports.Client = undefined;

var _get = function get(object, property, receiver) { if (object === null) object = Function.prototype; var desc = Object.getOwnPropertyDescriptor(object, property); if (desc === undefined) { var parent = Object.getPrototypeOf(object); if (parent === null) { return undefined; } else { return get(parent, property, receiver); } } else if ("value" in desc) { return desc.value; } else { var getter = desc.get; if (getter === undefined) { return undefined; } return getter.call(receiver); } };

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _lang = require('./lang');

var _lang2 = _interopRequireDefault(_lang);

var _lodash = require('lodash');

var _querystring = require('./querystring');

var _querystring2 = _interopRequireDefault(_querystring);

var _exceptions = require('./exceptions');

var _xml = require('./xml');

var _models = require('./models');

var _models2 = _interopRequireDefault(_models);

var _sort = require('./sort');

var _sort2 = _interopRequireDefault(_sort);

var _lru = require('./lru');

var _lru2 = _interopRequireDefault(_lru);

var _string = require('./string');

var _string2 = _interopRequireDefault(_string);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var integer = _lang.coerce.integer;

var P = Promise;

////////////////////////////////////////////////////////////////////////////////

/**
 * HTTP client
 */
var Client = exports.Client = function () {
  _createClass(Client, [{
    key: 'defaults',


    /**
     * Return instance configuration defaults
     * @param void
     * @return {Object}
     */
    value: function defaults() {
      var location = global.location || {
        protocol: 'https:',
        host: 'localhost'
      };

      return {
        language: 'en',

        // these must match your PrestaShop backend languages
        languages: {
          // ISO code => PrestaShop language id
          'en': 1
        },

        // API proxy configuration
        proxy: {
          scheme: location.protocol.slice(0, -1),
          host: location.host,
          root: '/shop/api'
        },

        // LRU instance; see https://www.npmjs.com/package/lru-cache
        cache: _lru2.default.instance(),

        // Fetch-related options
        fetch: {

          // the actual fetch function; facilitates testing
          algo: function algo() {
            var _global;

            if (!global.fetch) {
              return P.reject(new Error('fetch is not a global symbol'));
            }
            return (_global = global).fetch.apply(_global, arguments);
          },

          // Request options; see https://developer.mozilla.org/en-US/docs/Web/API/Request
          defaults: {
            cache: 'default',
            mode: 'same-origin'
          }
        }
      };
    }

    /**
     * @param {Object} options
     */

  }]);

  function Client() {
    var options = arguments.length <= 0 || arguments[0] === undefined ? {} : arguments[0];

    _classCallCheck(this, Client);

    this.options = _extends({}, this.defaults(), options);
    this.fetch = this.options.fetch.algo;
    this.cache = this.options.cache;
  }

  /**
   * Return a Prestashop API language id
   * @property
   * @type {Number}
   */


  _createClass(Client, [{
    key: 'get',


    /**
     * Send a GET request
     * @async Promise
     * @param {String} url
     * @param {Object} query
     * @return {Response}
     */
    value: function get(uri) {
      var _this = this;

      var options = arguments.length <= 1 || arguments[1] === undefined ? {} : arguments[1];

      var url = this.url(uri, options.query);
      var cachekey = this.options.language + ':' + url;
      var response = this.cache.get(cachekey);

      if (response) {
        return P.resolve(response.clone());
      }

      var fetchopts = _extends({}, this.options.fetch.defaults, options.fetch, {
        method: 'GET'
      });

      return this.fetch(url, fetchopts).then(function (response) {
        _this.validateResponse(response);
        _this.cache.set(cachekey, response);
        return response.clone();
      }).catch(function (e) {
        console.log(e.message + ' on url ' + url);
        throw e;
      });
    }

    /**
     * Return a fully qualified API url
     * @param {String} uri
     * @param {String}
     */

  }, {
    key: 'url',
    value: function url(uri) {
      var query = arguments.length <= 1 || arguments[1] === undefined ? {} : arguments[1];
      var proxy = this.options.proxy;

      var fullpath = _path2.default.join(proxy.root, uri);

      if (!_lang2.default.empty(query)) {
        query = _lang2.default.tuples(query).sort(_sort2.default.ascending(function (pair) {
          return pair[0];
        }));
        fullpath += '?' + _querystring2.default.stringify(query);
      }

      return proxy.scheme + '://' + proxy.host + fullpath;
    }

    /**
     * @param {Response} response
     * @return void
     * @throws Error
     */

  }, {
    key: 'validateResponse',
    value: function validateResponse(response) {
      if (!response.ok) {
        throw new _exceptions.UnexpectedValue('got non-2XX HTTP response');
      }
    }

    /**
     * @param {String} key
     * @param {Object} options
     * @return {rest.Resource}
     */

  }, {
    key: 'resource',
    value: function resource(key) {
      var options = arguments.length <= 1 || arguments[1] === undefined ? {} : arguments[1];

      var dict = {
        products: resources.Products,
        manufacturers: resources.Manufacturers,
        combinations: resources.Combinations,
        images: resources.Images
      };

      var constructor = dict[key];

      if (!constructor) {
        throw new _exceptions.InvalidArgument('invalid root resource: "' + key + '"');
      }

      return new constructor(_extends({}, options, { client: this }));
    }
  }, {
    key: 'language',
    get: function get() {
      var opts = this.options;
      return opts.languages[opts.language];
    }
  }]);

  return Client;
}();

////////////////////////////////////////////////////////////////////////////////
var resources = exports.resources = {};

/**
 * a REST resource
 */
var Resource = exports.Resource = resources.Resource = function () {
  _createClass(_class, [{
    key: 'defaults',


    /**
     * Return instance configuration defaults
     * @param void
     * @return {Object}
     */
    value: function defaults() {
      var classname = this.constructor.name;
      var api = _string2.default.snake(classname);
      var nodetype = api.slice(0, -1);
      var root = '/' + api;
      var modelname = classname.slice(0, -1);

      //console.log({api, nodetype, root, modelname});

      return {
        client: null,
        model: _models2.default[modelname],
        root: root,
        api: api,
        nodetype: nodetype,

        // model list filter function
        filter: null,

        // model list sort function
        sort: null
      };
    }

    /**
     * @param {Object} options
     */

  }], [{
    key: 'name',


    /**
     * Return the canonical constructor name. This is a workaround to a side
     * effect of babel transpilation, which is that babel mangles class names.
     * @property
     * @type {String}
     */
    get: function get() {
      return 'Resource';
    }
  }]);

  function _class() {
    var options = arguments.length <= 0 || arguments[0] === undefined ? {} : arguments[0];

    _classCallCheck(this, _class);

    this.options = _extends({}, this.defaults(), options);
    this.client = this.options.client;
  }

  /**
   * Return a PrestaShop language id
   * @property
   * @type {Number}
   */


  _createClass(_class, [{
    key: 'list',


    /**
     * Resolve an array of Model instances
     * @async Promise
     * @param void
     * @return {Array}
     */
    value: function list() {
      var _this2 = this;

      return this.client.get(this.options.root).then(function (response) {
        return response.text();
      }).then(function (xml) {
        return _this2.parseModelIds(xml);
      }).then(function (ids) {
        return _this2.createModels(ids);
      }).then(function (models) {
        var _options = _this2.options;
        var sort = _options.sort;
        var filter = _options.filter;


        if (filter) {
          models = models.filter(filter);
        }
        if (sort) {
          models = models.sort(sort);
        }

        return models;
      });
    }

    /**
     * Return the first member of a list() result
     * @param void
     * @return {models.Model}
     */

  }, {
    key: 'first',
    value: function first() {
      return this.list().then(function (models) {
        return models[0] || null;
      });
    }

    /**
     * Resolve a single Model instance
     * @async Promise
     * @param {mixed} id
     * @return {Model}
     */

  }, {
    key: 'get',
    value: function get(id) {
      var _this3 = this;

      return this.client.get(this.options.root + '/' + id).then(function (response) {
        return response.text();
      }).then(function (xml) {
        return _this3.parseModelProperties(xml);
      }).then(function (props) {
        return _this3.createModel(props);
      });
    }

    /**
     * Map a list of model ids to model instances.
     * @async Promise
     * @param {Array} ids
     * @return {Array}
     */

  }, {
    key: 'createModels',
    value: function createModels(ids) {
      var _this4 = this;

      return P.all(ids.map(function (id) {
        return _this4.get(id);
      }));
    }

    /**
     * Given an object containing model properties, return a Model instance.
     * @param {Object} props
     * @return {Model}
     */

  }, {
    key: 'createModel',
    value: function createModel(props) {
      var constructor = this.options.model;


      return new constructor({
        client: this.client,
        resource: this,
        props: props
      });
    }

    /**
     * Given the API response payload for a list of objects, return a list of
     * model ids.
     * @async Promise
     * @param {String} xml
     * @return {Array}
     */

  }, {
    key: 'parseModelIds',
    value: function parseModelIds(xml) {
      var _options2 = this.options;
      var api = _options2.api;
      var nodetype = _options2.nodetype;

      return _xml.parse.model.ids(xml, api, nodetype);
    }

    /**
     * Given the API response payload for a single domain object, return a plain
     * object that contains model properties.
     * @param {String} xml
     * @return {Object}
     */

  }, {
    key: 'parseModelProperties',
    value: function parseModelProperties(xml) {
      var nodetype = this.options.nodetype;
      var ns = _xml.parse[nodetype];

      if (!ns) {
        throw new _exceptions.UnexpectedValue('parser namespace not found on node type ' + nodetype);
      }
      if (!ns.properties) {
        throw new _exceptions.UnexpectedValue('model properties parser not found on node type ' + nodetype);
      }

      return _xml.parse[nodetype].properties(xml, this.language);
    }
  }, {
    key: 'language',
    get: function get() {
      return this.client.language;
    }
  }]);

  return _class;
}();

resources.Products = function (_Resource) {
  _inherits(_class2, _Resource);

  function _class2() {
    _classCallCheck(this, _class2);

    return _possibleConstructorReturn(this, (_class2.__proto__ || Object.getPrototypeOf(_class2)).apply(this, arguments));
  }

  _createClass(_class2, null, [{
    key: 'name',


    /**
     * inheritdoc
     */
    get: function get() {
      return 'Products';
    }
  }]);

  return _class2;
}(Resource);

resources.Images = function (_Resource2) {
  _inherits(_class3, _Resource2);

  function _class3() {
    _classCallCheck(this, _class3);

    return _possibleConstructorReturn(this, (_class3.__proto__ || Object.getPrototypeOf(_class3)).apply(this, arguments));
  }

  _createClass(_class3, [{
    key: 'list',


    /**
     * @inheritdoc
     */
    value: function list() {
      var _this7 = this;

      return this.client.get(this.options.root).then(function (response) {
        return response.text();
      }).then(function (xml) {
        return _this7.parseImageProperties(xml);
      }).then(function (propsets) {
        return propsets.map(function (props) {
          return _this7.createModel(props);
        });
      });
    }

    /**
     * Return a list of urls given an XML payload
     * @async Promise
     * @param {String} xml
     * @return {Array}
     */

  }, {
    key: 'parseImageProperties',
    value: function parseImageProperties(xml) {
      return _xml.parse.image.properties(xml);
    }
  }], [{
    key: 'name',
    get: function get() {
      return 'Images';
    }
  }]);

  return _class3;
}(Resource);

resources.Manufacturers = function (_Resource3) {
  _inherits(_class4, _Resource3);

  function _class4() {
    _classCallCheck(this, _class4);

    return _possibleConstructorReturn(this, (_class4.__proto__ || Object.getPrototypeOf(_class4)).apply(this, arguments));
  }

  _createClass(_class4, null, [{
    key: 'name',


    /**
     * @inheritdoc
     */
    get: function get() {
      return 'Manufacturers';
    }
  }]);

  return _class4;
}(Resource);

resources.Combinations = function (_Resource4) {
  _inherits(_class5, _Resource4);

  function _class5() {
    _classCallCheck(this, _class5);

    return _possibleConstructorReturn(this, (_class5.__proto__ || Object.getPrototypeOf(_class5)).apply(this, arguments));
  }

  _createClass(_class5, null, [{
    key: 'name',


    /**
     * @inheritdoc
     */
    get: function get() {
      return 'Combinations';
    }
  }]);

  return _class5;
}(Resource);

resources.StockAvailables = function (_Resource5) {
  _inherits(_class6, _Resource5);

  function _class6() {
    _classCallCheck(this, _class6);

    return _possibleConstructorReturn(this, (_class6.__proto__ || Object.getPrototypeOf(_class6)).apply(this, arguments));
  }

  _createClass(_class6, null, [{
    key: 'name',


    /**
     * @inheritdoc
     */
    get: function get() {
      return 'StockAvailables';
    }
  }]);

  return _class6;
}(Resource);

resources.ProductOptionValues = function (_Resource6) {
  _inherits(_class7, _Resource6);

  function _class7() {
    _classCallCheck(this, _class7);

    return _possibleConstructorReturn(this, (_class7.__proto__ || Object.getPrototypeOf(_class7)).apply(this, arguments));
  }

  _createClass(_class7, [{
    key: 'defaults',


    /**
     * @inheritdoc
     */
    value: function defaults() {
      return _extends({}, _get(_class7.prototype.__proto__ || Object.getPrototypeOf(_class7.prototype), 'defaults', this).call(this), {
        sort: _sort2.default.ascending(function (model) {
          return model.position;
        })
      });
    }
  }], [{
    key: 'name',


    /**
     * inheritdoc
     */
    get: function get() {
      return 'ProductOptionValues';
    }
  }]);

  return _class7;
}(Resource);

exports.default = { Client: Client, resources: resources };